SystemOrganization addCategory: #'Rest-Client-HTTP'!
SystemOrganization addCategory: #'Rest-Client-Proxies'!

Object subclass: #RestObject
	instanceVariableNames: 'properties isInitialized socket baseUri canonicalName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestObject class methodsFor: 'as yet unclassified' stamp: 'SR 2/26/2009 15:19'!
newFromBase: aBaseUriString with: aPathString
	| data |

	data := Json readFrom: (RestSocket get: (aBaseUriString, aPathString)) content readStream.
	^ data isDictionary
		ifTrue: [
			self new
				canonicalName: aPathString;
				baseUri: aBaseUriString;
				reload]
		ifFalse: [ |response|
			response := OrderedCollection new.
			data do: [:each |
				response add: (RestObject newFromBase: aBaseUriString with: each)].
			response]! !

!RestObject methodsFor: 'accessing' stamp: 'SR 2/26/2009 15:18'!
baseUri
	^ baseUri! !

!RestObject methodsFor: 'accessing' stamp: 'SR 2/26/2009 15:18'!
baseUri: anObject
	baseUri := anObject! !

!RestObject methodsFor: 'accessing' stamp: 'SR 2/26/2009 15:18'!
canonicalName
	^ canonicalName! !

!RestObject methodsFor: 'accessing' stamp: 'SR 2/26/2009 15:18'!
canonicalName: anObject
	canonicalName := anObject! !

!RestObject methodsFor: 'error handling' stamp: 'topa 2/24/2009 18:54'!
doesNotUnderstand: aMessage
	^ (self properties includesKey: aMessage asString)
		ifTrue: [self properties at: aMessage asString]
		ifFalse: [super doesNotUnderstand: aMessage]! !

!RestObject methodsFor: 'accessing' stamp: 'topa 2/24/2009 18:53'!
properties
	^ properties! !

!RestObject methodsFor: 'accessing' stamp: 'mig 2/23/2009 16:54'!
properties: anObject
	properties := anObject! !

!RestObject methodsFor: 'as yet unclassified' stamp: 'SR 2/26/2009 15:17'!
reload
	self properties: (Json readFrom: (RestSocket get: (self baseUri, self canonicalName)) content readStream).! !

!RestObject methodsFor: 'accessing' stamp: 'topa 2/25/2009 11:08'!
socket
	^ socket! !

!RestObject methodsFor: 'accessing' stamp: 'topa 2/25/2009 11:08'!
socket: anObject
	socket := anObject! !

Object subclass: #RestObjectFactory
	instanceVariableNames: 'uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestObjectFactory class methodsFor: 'as yet unclassified' stamp: 'cs 2/27/2009 11:35'!
newFrom: aString

	^ self new uri: (SwazooURI fromString: aString)! !

!RestObjectFactory methodsFor: 'instance creation' stamp: 'cs 2/27/2009 12:30'!
addObject: path with: anObject
	| stream response |
	stream := TextStream with: ''.
	anObject jsonWriteOn: stream.
	response := RestSocket
		postDocumentFromUrl: (self uri copy identifier: self uri identifier, path) asString asUrl
		request: ''
		content: stream contents.
	^ RestObject
		newFromBase: self uri asString
		with: response content! !

!RestObjectFactory methodsFor: 'instance creation' stamp: 'cs 2/27/2009 12:15'!
getObject: path

	^ RestObject
		newFromBase: uri asString
		with: path! !

!RestObjectFactory methodsFor: 'accessing' stamp: 'cs 2/27/2009 11:35'!
uri
	^ uri! !

!RestObjectFactory methodsFor: 'accessing' stamp: 'cs 2/27/2009 11:35'!
uri: anObject
	uri := anObject! !

Object subclass: #RestResponse
	instanceVariableNames: 'content headers responseCode url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestResponse class methodsFor: 'as yet unclassified' stamp: 'mfp 2/20/2009 16:05'!
responseCode: aNumberCode headers: aDictionary content: aString url: aUrl

	^ self new
		responseCode: aNumberCode asNumber;
		headers: aDictionary;
		content: aString;
		url: aUrl! !

!RestResponse methodsFor: 'accessing' stamp: 'SR 2/19/2009 16:01'!
content
	^ content! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:45'!
content: anObject
	content := anObject! !

!RestResponse methodsFor: 'accessing' stamp: 'SR 2/19/2009 16:03'!
getHeader: name 
	^self getHeader: name  default: nil! !

!RestResponse methodsFor: 'accessing' stamp: 'SR 2/19/2009 16:03'!
getHeader: name  default: defaultValue
	^headers at: name  ifAbsent: [ defaultValue ]! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:45'!
headers
	^ headers! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:45'!
headers: anObject
	headers := anObject! !

!RestResponse methodsFor: 'accessing' stamp: 'SR 2/19/2009 16:03'!
responseCode
	^ responseCode! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:46'!
responseCode: anObject
	responseCode := anObject! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:46'!
url
	^ url! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:46'!
url: anObject
	url := anObject! !

Object subclass: #RestSocket
	instanceVariableNames: 'conn headers responseCode request content url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestSocket class methodsFor: 'private' stamp: 'SR 2/19/2009 16:58'!
defaultPortForSchema: aSchema
	"default port to connect on"
	(aSchema = 'http') ifTrue: [^ 80].
	(aSchema = 'https') ifTrue: [^ 443].! !

!RestSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 16:15'!
deleteDocumentFromUrl: url request: requestString 
	^ self 
		doHTTPRequestFromUrl: url
		verb: 'DELETE'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'get the page' stamp: 'topa 2/25/2009 11:11'!
doHTTPRequestFromUrl: url verb: aVerb request: requestString content: aContentString 
	| serverName sock |
	self flag: #NeedsRefactoring.
	Socket initializeNetwork.

	serverName := url authority.
	3 timesRepeat: [
		sock := self new 
			url: url;
			request: requestString;
			content: aContentString.
		(sock connectAndWaitUntil: (Time millisecondClockValue + 10000 truncated))
			ifTrue: [^ sock doVerb: aVerb]].
	Socket deadServer: serverName.
	sock destroy.
	^ 'Server ' , serverName , ' is not responding'! !

!RestSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 14:07'!
expandUrl: newUrl ip: byteArrayIP port: portNum

	(newUrl beginsWith: '../') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: (newUrl allButFirst: 2)]].
	(newUrl beginsWith: '/') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: newUrl]].
	^ newUrl
! !

!RestSocket class methodsFor: 'get the page' stamp: 'SR 2/26/2009 14:59'!
get: aUriString
	"Send an HTTP GET request."

	^ RestSocket 
		getDocumentFromUrl: aUriString asUrl "for Socket compatibility"
		request: ''! !

!RestSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 16:15'!
getDocumentFromUrl: url request: requestString 
	^ self 
		doHTTPRequestFromUrl: url
		verb: 'GET'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 16:15'!
postDocumentFromUrl: url request: requestString content: contentString 
	^ self 
		doHTTPRequestFromUrl: url
		verb: 'POST'
		request: requestString
		content: contentString! !

!RestSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 16:15'!
putDocumentFromUrl: url request: requestString content: contentString 
	^ self 
		doHTTPRequestFromUrl: url
		verb: 'PUT'
		request: requestString
		content: contentString! !

!RestSocket methodsFor: 'accessing' stamp: 'SR 2/19/2009 14:07'!
conn
	"Answer the value of conn"

	^ conn! !

!RestSocket methodsFor: 'accessing' stamp: 'SR 2/19/2009 14:07'!
conn: anObject
	"Set the value of conn"

	conn := anObject! !

!RestSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:41'!
connect
	| serverAddr |
	serverAddr := (NetNameResolver addressForName: self serverName timeout: 20)
		ifNil: [self error: 'Could not resolve the server named: ', self serverName].
		
	self connectTo: serverAddr port: self port! !

!RestSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:41'!
connectAndWaitUntil: aTimeout

	^ self connect; waitForConnectionUntil: aTimeout! !

!RestSocket methodsFor: 'api' stamp: 'cs 2/19/2009 20:33'!
connectTo: addr port: aPort

	self conn:
		(self usesSSL
			ifTrue: [SSLSocket newConnectionAddress: addr port: aPort version: SSLProtocolVersion tls10]
			ifFalse: [Socket new connectTo: addr port: aPort]
		).
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:12'!
content
	^ content! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:12'!
content: anObject
	content := anObject! !

!RestSocket methodsFor: 'api' stamp: 'SR 2/19/2009 14:07'!
destroy

	self conn close! !

!RestSocket methodsFor: 'api' stamp: 'mig 2/23/2009 16:45'!
doVerb: aHTTPVerb
	| cmd stream |
	cmd := 	aHTTPVerb,' ', self url fullPath, ' HTTP/1.0', String crlf, 
				'Host: ', self serverName , ':', self port printString, String crlf, 
				(self request ifEmpty: [''] ifNotEmpty: [self request, String crlf]).
					
	self content ifNotNil: [
		cmd := cmd, 
					'Content-type: application/json', String crlf,
					'Content-length: ', self content size printString , String crlf].
	self sendCommand: cmd.
	"headers are sent."					
	self content ifNotNil: [self sendCommand: self content].
	stream := self getData readStream.
	self headers: (stream upToAll: String crlf , String crlf). "parses header, too"
	^ RestResponse 
			responseCode: self responseCode  
			headers: self headers
			content: (self getRestOfData: stream upToEnd)
			url: self url! !

!RestSocket methodsFor: 'api' stamp: 'cs 2/19/2009 21:57'!
getData

	| data |
	data := self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveData].
	[self conn isConnected and: [(data asString findString: String crlfcrlf) == 0]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveData]).].
	^ data asString! !

!RestSocket methodsFor: 'api' stamp: 'SR 2/19/2009 14:07'!
getHeader: name 
	^self getHeader: name  default: nil! !

!RestSocket methodsFor: 'api' stamp: 'SR 2/19/2009 14:07'!
getHeader: name  default: defaultValue
	^headers at: name  ifAbsent: [ defaultValue ]! !

!RestSocket methodsFor: 'api' stamp: 'cs 2/19/2009 22:06'!
getRestOfData: begin

	| data length |
	length := self getHeader: 'content-length'.
	length ifNotNil: [ length := length asNumber ].
	data := begin, (self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveAvailableData]).
	[self conn isConnected and: [length isNil or: [data size < length]]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveAvailableData]).].
	^ data asString! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 19:52'!
headers
	^ headers! !

!RestSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:53'!
headers: headerText
	"set the headers.  Then getHeader: can be used"

	"divide into basic lines"
	| lines foldedLines i statusLine |
	lines := headerText findTokens: (String with: Character cr  with: Character linefeed).
	statusLine := lines first.
	lines := lines copyFrom: 2 to: lines size.

	"parse the status (pretty trivial right now)"
	responseCode := (statusLine findTokens: ' ') second.

	"fold lines that start with spaces into the previous line"
	foldedLines := OrderedCollection new.
	lines do: [ :line |
		line first isSeparator ifTrue: [
			foldedLines at: foldedLines size  put: (foldedLines last, line) ]
		ifFalse: [ foldedLines add: line ] ].

	"make a dictionary mapping headers to header contents"
	headers := Dictionary new.
	foldedLines do: [ :line |
		i := line indexOf: $:.
		i > 0 ifTrue: [
			self headers 
			at: (line copyFrom: 1 to: i-1) asLowercase 
			put: (line copyFrom: i+1 to: line size) withBlanksTrimmed ] ].
! !

!RestSocket methodsFor: 'accessing' stamp: 'cs 2/19/2009 20:34'!
isConnecting
	^ self usesSSL
		ifTrue: [self conn isConnecting ]
		ifFalse: [self conn isWaitingForConnection ]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 19:18'!
port
	^ 	self url port 
		ifNil:  [self class defaultPortForSchema: self url schemeName]
		ifNotNil: [self url port]
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:12'!
request
	^ request! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:12'!
request: anObject
	request := anObject! !

!RestSocket methodsFor: 'api' stamp: 'SR 2/19/2009 14:07'!
responseCode
	^responseCode! !

!RestSocket methodsFor: 'api' stamp: 'cs 2/19/2009 20:35'!
sendCommand: cmd
	
	self usesSSL
		ifTrue: [self conn send: cmd, String crlf.]
		ifFalse: [self conn sendData: cmd, String crlf.]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:58'!
serverName
	^ self url authority! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:23'!
url
	^ url! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:23'!
url: anObject
	url := anObject! !

!RestSocket methodsFor: 'accessing' stamp: 'test 2/19/2009 20:42'!
usesSSL
	^ url schemeName = 'https'! !

!RestSocket methodsFor: 'api' stamp: 'cs 2/19/2009 20:33'!
waitForConnectionUntil: deadline
	"Wait up until the given deadline for a connection to be established. Return true if it is established by the deadline, false if not."

	[(self isConnecting) and: [Time millisecondClockValue < deadline]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait].
	^ self conn isConnected
! !
